#!/usr/bin/env python
# -*- coding: utf-8 -*-
intro = \
"""
ReadSCPI_18c.py
ProfHuster
2018-02-05

This program collects data from an Arduino (or Arduino-clone) that uses
a SCPI-style command structure and saves it to a file.

Versions:
18c - Various mod’s including saving CONF? query to the data file
18b - Flushes stdout
"""
print intro
import serial, io, re, sys, time
from serialPorts import serial_ports

# The next line defines the file name format
# Note that the format automatically include the 
# year-month-date-hour-minute. This is incredibly useful to help
# keep data files orgnized. The alphabetical order of the files
# is also the chrononlogical order.
FILE_NAME_FORMAT = r"SCPI-%04d-%02d-%02d-%02d%02d.csv"
EOL = '\n'
TIMEOUT = 1.0
DO_PRINT = True # print everything out as you go
DO_PRINT = False

# make file name and open output data file
t = time.localtime()
fileName = FILE_NAME_FORMAT % (t.tm_year, t.tm_mon, t.tm_mday, \
    t.tm_hour, t.tm_min)
print 'Opening file "%s"' % (fileName)
fpData = open(fileName, 'w')

# Let user put in comments
while True:
    comment = raw_input("Enter a comment (return to end): ")
    if len(comment.strip()) == 0:
        break
    fpData.write("# " + comment.strip()+'\n')
fpData.flush()

# First get the name of and open the serial port
ports = serial_ports()
print "The available ports are:"
for (i, port) in enumerate(ports):
	print "%d, %s" % (i, port)
iPort = int(raw_input("Enter serial port number: "))
#select serial port, baudrate (default = 9600), timeout (default = 1)
ser = serial.Serial(ports[iPort], 115200, timeout=TIMEOUT)

# wait for Arduino to boot and print any output
time.sleep(TIMEOUT)

# This code is used later also
# Read a line form Arduino
line = ser.readline()
# While line has some length. If length is zero, there is no more to read
while len(line) > 0:
    if DO_PRINT: print ">%s<" % (line.strip())
    line = ser.readline()

print "Enter command line (help? for help)"
print "Enter 'q' to quit"
# Loop until a break statement
while True:
    # Get line to write
    command = raw_input(": ")
    if DO_PRINT: print "Command:%s:" % (command.strip())
    # Check if a q for quit was entered
    if len(command)>0 and command[0].lower() == 'q':
        break
    # Add a newline before sending command to Arduino
    command += EOL
    # If it is a data command, send config query and write response to the data file
    if re.search(r'data?', command, flags=re.I):
        ser.write('conf?\n')
        line = ser.readline()
        while len(line) > 0:
            if DO_PRINT: print ">%s<" % (line.strip())
            sys.stdout.flush()
            re.sub(r'^', r'# ', line)
            line = re.sub(r'$', r'', re.sub(r'<', '', re.sub(r'>', r'', re.sub(r'^', r'# ', line))))
            fpData.write(line)
            fpData.flush()
            line = ser.readline()
    ser.write(command)

    # look for the data command. If found, save to the file
    if re.search(r'data?', command, flags=re.I):
        # Read and print first line
        line = ser.readline()
        if DO_PRINT: print ">%s<" % (line.strip())
        # Now save the data output
        line = ser.readline()
        while len(line) > 0:
            if DO_PRINT: print ">%s<" % (line.strip())
            sys.stdout.flush()
            fpData.write(line.strip()+'\n')
            fpData.flush()
            line = ser.readline()
        if not DO_PRINT: print "Done"
    else:
        # skip one line
        line = ser.readline()
        for line in ser.readlines():
            lineStrip = line.strip()
            print ">%s<" % (lineStrip)
ser.close()
fpData.close()
